/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.common;

import java.util.Map;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;

/**
 * A marker output is used in an Eclipse environment to report validation
 * messages to the problem view 
 * 
 * @author Ali Mehregani
 */
public class MarkerOutput extends AbstractValidationOutput
{
	/**
	 * The ID of the Service Modeling Language marker
	 */
	public static final String SML_MARKER_ID = "org.eclipse.cosmos.rm.validation.smlMarker";
	
	/**
	 * The worksapce root
	 */
	private IWorkspaceRoot workspaceRoot;
	
	/**
	 * The attributes describing the settings
	 */
	private Map settings;
	
	/**
	 * Used as the backup validation output
	 */
	private FileOutput backupOutput;
		
	/**
	 * Indicates if an error message has already been reported
	 */
	private boolean errorReported;
	
	public MarkerOutput()
	{
		workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();		
		errorReported = false;
	}	
	
	/** 
	 * @see org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput#initialize(java.util.Map)
	 */
	public void initialize(Map attributes)
	{
		this.settings = attributes;
	}	
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput#reportMessage(org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput.ValidationMessage)
	 */
	public void reportMessage(ValidationMessage validationMessage)
	{
		IMarker marker;
		try
		{			
			Object rsc = validationMessage.getAttribute(ValidationMessage.ATTRIBUTE_RESOURCE);
			IResource resource = rsc instanceof IResource ? (IResource)rsc : workspaceRoot;
			int severity = validationMessage.getAttribute(ValidationMessage.ATTRIBUTE_SEVERITY, ValidationMessage.SEVERITY_INFO);
			marker = resource.createMarker(SML_MARKER_ID);
			marker.setAttribute(IMarker.MESSAGE, validationMessage.getAttribute(ValidationMessage.ATTRIBUTE_MESSAGE, IValidationConstants.EMPTY_STRING));
			marker.setAttribute(IMarker.SEVERITY, severity == ValidationMessage.SEVERITY_ERROR ? IMarker.SEVERITY_ERROR :
												 (severity == ValidationMessage.SEVERITY_WARNING ? IMarker.SEVERITY_WARNING : IMarker.SEVERITY_INFO));
			
			int lineNumber = validationMessage.getAttribute(ValidationMessage.ATTRIBUTE_LINE_NUMBER, -1);
			if (lineNumber > 0)
			{
				marker.setAttribute(IMarker.LINE_NUMBER, lineNumber);
			}
		} 
		catch (CoreException e)
		{
			if (backupOutput == null)
			{
				backupOutput = new FileOutput();
				backupOutput.initialize(settings);
			}
			
			if (!errorReported)
			{
				errorReported = true;
				MessageDialog.openError(SMLActivator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
						SMLMessages.commonError, NLS.bind(SMLMessages.errorMarkerOutput, backupOutput.getFilePath()));
			}
			
			backupOutput.reportMessage(validationMessage);
		}

		
	}
}
