/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Abstract class that provides a default implementation of the IDataBuilder interface.
 * 
 * @author sleeloy
 * @author Ali Mehregani
 */
public abstract class AbstractDataBuilder extends DefaultHandler implements IDataBuilder 
{	
	/** 
	 * Stores line and column information
	 */
	protected Locator locator;
	
	/**
	 * The target namespace
	 */
	private String targetNamespace;
	
	/**
	 * The element/attribute prefix
	 */
	private String prefix;
	
	/**
	 * The validity of the structure generated.  Set to true by
	 * default.
	 */
	private boolean validity;
	
	/**
	 * The error message
	 */
	private String errorMessage;
	
	/**
	 * Constructor
	 */
	public AbstractDataBuilder()
	{
		setStructureValidity(true);
		targetNamespace = "";
		prefix = "";
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#endGenics()
	 */
	public void endGenics() 
	{
		// no-op
	}

	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#endPhenics()
	 */
	public void endPhenics() 
	{
		// no-op
	}

	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#getPhase()
	 */
	public byte getPhase() 
	{
		return ISMLConstants.GENICS_PHENICS_PHASE;
	}

	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#startGenics()
	 */
	public void startGenics() 
	{
		// no-op
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#startPhenics()
	 */
	public void startPhenics() 
	{
		// no-op
	}
	

	
	/**
	 * This is used to determine the target name space of the document currently
	 * being parsed
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		if (ISMLConstants.SCHEMA_ELEMENT.equals(localName))
		{
			targetNamespace = attributes.getValue(ISMLConstants.TARGET_NAMESPACE_ATTRIBUTE);
			if (targetNamespace == null)
				return;
			for (int i = 0, attributeCount = attributes.getLength(); i < attributeCount; i++)
			{
				if (!ISMLConstants.TARGET_NAMESPACE_ATTRIBUTE.equals(attributes.getQName(i)) && 
					targetNamespace.equals(attributes.getValue(i)))
				{
					String attributeName = attributes.getQName(i);
					int columnInx = attributeName.indexOf(':');					
					if (columnInx >= 0)
						prefix = attributeName.substring(columnInx + 1) + ":";
				}
			}
		}
	
	}
	
	/**
	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		if (ISMLConstants.SCHEMA_ELEMENT.equals(localName))
		{
			targetNamespace = "";
			prefix = "";
		}
	}
	
	
	/**
	 * @see org.xml.sax.helpers.DefaultHandler#setDocumentLocator(org.xml.sax.Locator)
	 */
	public void setDocumentLocator(Locator locator) 
	{
		this.locator = locator;
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#isStructureValid()
	 */
	public boolean isStructureValid()
	{		
		return validity;
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#setStructureValidity(boolean)
	 */
	public void setStructureValidity(boolean validity)
	{
		this.validity = validity;
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#getErrorMessage()
	 */
	public String getErrorMessage()
	{
		return errorMessage;
	}	
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#setErrorMessage(java.lang.String)
	 */
	public void setErrorMessage(String error)
	{
		this.errorMessage = error;
	}
		
	/**
	 * Appends to the error message
	 * 
	 * @param error The error, which is appended as a separate line
	 */
	protected void appendToErrorMessage(String error)
	{
		if (errorMessage == null)
			errorMessage = "";
		
		if (errorMessage.length() > 0)
			errorMessage += IValidationConstants.LINE_SEPARATOR;
		
		errorMessage += error;
	}

	/**
	 * @return the prefix
	 */
	public String getPrefix()
	{
		return prefix;
	}

	/**
	 * @param prefix the prefix to set
	 */
	public void setPrefix(String prefix)
	{
		this.prefix = prefix;
	}

	/**
	 * @return the targetNamespace
	 */
	public String getTargetNamespace()
	{
		return targetNamespace;
	}

	/**
	 * @param targetNamespace the targetNamespace to set
	 */
	public void setTargetNamespace(String targetNamespace)
	{
		this.targetNamespace = targetNamespace;
	}
}
