/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.Hashtable;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * This abstract property extractor will extract a set of elements/attribute values
 * based on a set of conditions specified (e.g. the value of the name attribute for elements
 * that have acyclic set to true)
 * 
 * @author Ali Mehregani
 */
public class AbstractPropertyExtractor extends AbstractDataBuilder implements IDataBuilder
{
	/**
	 * The phase that this builder should operate in
	 */
	private byte phase;
	
	/**
	 * The element name
	 */
	private String elementName;
	
	/**
	 * An array of String[] of length 2, where string[0] = attribute name
	 * string[1] = expected attribute value.
	 */
	private String[][] attributes;
	
	/**
	 * The key/value pair to store
	 * Must be of length 2
	 */
	private String[] keyValuePair;
	
	/**
	 * The list of elements stored that meet the required condition
	 */
	private Map entries;

	/**
	 * Indicates whether a prefix should be used as part of storing the value
	 * of an attribute. 
	 */
	private boolean usePrefix;
	
	
	/**
	 * Constructor 
	 * 
	 * @param phase The phase that this builder should operate in (see ISMLConstants.*PHASE)
	 * @param elementName The expected element name
	 * @param attributes The expected attribute name and values
	 * @param keyValuePair The key/value pair to store.   Must be of length 2
	 */
	public AbstractPropertyExtractor(byte phase, String elementName, String[][] attributes, String[] keyValuePair)
	{
		this.phase = phase;
		this.elementName = elementName;
		this.attributes = attributes;
		this.keyValuePair = keyValuePair;
		entries = new Hashtable();
	}
	
	/**
	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{		
		super.startElement(uri, localName, qName, attributes);
		if (elementName != null && (!elementName.equals(localName)))
			return;
		
		if (this.attributes != null)
		{
			for (int i = 0; i < this.attributes.length; i++)
			{
				String currentAttributeValue = attributes.getValue(this.attributes[i][0]);
				if (currentAttributeValue == null)
					return;
				if (this.attributes[i][1] != null && !this.attributes[i][1].equals(currentAttributeValue))
					return;
			}
		}
		
		String valueToStore = attributes.getValue(keyValuePair[0]);
		if (valueToStore != null && entries.get(valueToStore) == null)
			entries.put((usePrefix ? getPrefix() : "") + valueToStore, keyValuePair[1] == null ? IValidationConstants.TRUE_VALUE : attributes.getValue(keyValuePair[1]));
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#getDataStructure()
	 */
	public Object getDataStructure()
	{
		return entries.keySet().iterator();
	}

	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.AbstractDataBuilder#getPhase()
	 */
	public byte getPhase() 
	{
		return phase;
	}

	/**
	 * Returns the map version of the entries stored
	 * 
	 * @return The map version of the entries stored
	 */
	protected Object getMapDataStructure()
	{
		return entries;
	}

	public void setUsePrefix(boolean usePrefix)
	{
		this.usePrefix = usePrefix;
	}
	
	public boolean getUsePrefix()
	{
		return usePrefix;
	}
}
